@@ -1,4 +1,5 @@
 from dagster import AssetKey, DagsterEventType, EventRecordsFilter, check, seven
+from dagster.core.events import ASSET_EVENTS
 
 from .utils import capture_error
 
@@ -41,7 +42,10 @@ def get_assets(graphene_info, prefix=None, cursor=None, limit=None):
 
     return GrapheneAssetConnection(
         nodes=[
-            GrapheneAsset(key=asset_key, definition=asset_nodes_by_asset_key.get(asset_key))
+            GrapheneAsset(
+                key=asset_key,
+                definition=asset_nodes_by_asset_key.get(asset_key),
+            )
             for asset_key in asset_keys
         ]
     )
@@ -51,7 +55,7 @@ def get_asset_nodes_by_asset_key(graphene_info):
     from ..schema.asset_graph import GrapheneAssetNode
 
     return {
-        external_asset_node.asset_key: GrapheneAssetNode(repository, external_asset_node)
+        external_asset_node.asset_key: GrapheneAssetNode(location, repository, external_asset_node)
         for location in graphene_info.context.repository_locations
         for repository in location.get_repositories().values()
         for external_asset_node in repository.get_external_asset_nodes()
@@ -62,7 +66,7 @@ def get_asset_nodes(graphene_info):
     from ..schema.asset_graph import GrapheneAssetNode
 
     return [
-        GrapheneAssetNode(repository, external_asset_node)
+        GrapheneAssetNode(location, repository, external_asset_node)
         for location in graphene_info.context.repository_locations
         for repository in location.get_repositories().values()
         for external_asset_node in repository.get_external_asset_nodes()
@@ -73,7 +77,7 @@ def get_asset_node(graphene_info, asset_key):
     from ..schema.errors import GrapheneAssetNotFoundError
 
     check.inst_param(asset_key, "asset_key", AssetKey)
-    node = next((n for n in get_asset_nodes(graphene_info) if n.assetKey == asset_key), None)
+    node = get_asset_nodes_by_asset_key(graphene_info).get(asset_key, None)
     if not node:
         return GrapheneAssetNotFoundError(asset_key=asset_key)
     return node
@@ -92,11 +96,17 @@ def get_asset(graphene_info, asset_key):
     if not asset_node and not instance.has_asset_key(asset_key):
         return GrapheneAssetNotFoundError(asset_key=asset_key)
 
-    tags = instance.get_asset_tags(asset_key)
-    return GrapheneAsset(key=asset_key, definition=asset_node, tags=tags)
+    return GrapheneAsset(key=asset_key, definition=asset_node)
 
 
-def get_asset_events(graphene_info, asset_key, partitions=None, limit=None, before_timestamp=None):
+def get_asset_materializations(
+    graphene_info,
+    asset_key,
+    partitions=None,
+    limit=None,
+    before_timestamp=None,
+    after_timestamp=None,
+):
     check.inst_param(asset_key, "asset_key", AssetKey)
     check.opt_int_param(limit, "limit")
     check.opt_float_param(before_timestamp, "before_timestamp")
@@ -107,6 +117,33 @@ def get_asset_events(graphene_info, asset_key, partitions=None, limit=None, befo
             asset_key=asset_key,
             asset_partitions=partitions,
             before_timestamp=before_timestamp,
+            after_timestamp=after_timestamp,
+        ),
+        limit=limit,
+    )
+    return [event_record.event_log_entry for event_record in event_records]
+
+
+def get_asset_observations(
+    graphene_info,
+    asset_key,
+    partitions=None,
+    limit=None,
+    before_timestamp=None,
+    after_timestamp=None,
+):
+    check.inst_param(asset_key, "asset_key", AssetKey)
+    check.opt_int_param(limit, "limit")
+    check.opt_float_param(before_timestamp, "before_timestamp")
+    check.opt_float_param(after_timestamp, "after_timestamp")
+    instance = graphene_info.context.instance
+    event_records = instance.get_event_records(
+        EventRecordsFilter(
+            event_type=DagsterEventType.ASSET_OBSERVATION,
+            asset_key=asset_key,
+            asset_partitions=partitions,
+            before_timestamp=before_timestamp,
+            after_timestamp=after_timestamp,
         ),
         limit=limit,
     )
@@ -124,7 +161,7 @@ def get_assets_for_run_id(graphene_info, run_id):
 
     check.str_param(run_id, "run_id")
 
-    records = graphene_info.context.instance.all_logs(run_id)
+    records = graphene_info.context.instance.all_logs(run_id, of_type=ASSET_EVENTS)
     asset_keys = [
         record.dagster_event.asset_key
         for record in records
