@@ -4,9 +4,12 @@ from dagster.core.events import AssetLineageInfo, DagsterEventType
 from dagster.core.execution.plan.objects import ErrorSource
 from dagster.core.execution.stats import RunStepKeyStatsSnapshot
 
-from ...implementation.fetch_runs import get_step_stats
+from ...implementation.events import construct_basic_params
+from ...implementation.fetch_runs import get_run_by_id, get_step_stats
+from ...implementation.loader import BatchRunLoader
 from ..asset_key import GrapheneAssetKey, GrapheneAssetLineageInfo
-from ..errors import GraphenePythonError
+from ..errors import GraphenePythonError, GrapheneRunNotFoundError
+from ..metadata import GrapheneMetadataEntry
 from ..runs import GrapheneStepEventStatus
 from ..util import non_null_list
 from .log_level import GrapheneLogLevel
@@ -25,18 +28,10 @@ class GrapheneMessageEvent(graphene.Interface):
         name = "MessageEvent"
 
 
-class GrapheneEventMetadataEntry(graphene.Interface):
-    label = graphene.NonNull(graphene.String)
-    description = graphene.String()
-
-    class Meta:
-        name = "EventMetadataEntry"
-
-
 class GrapheneDisplayableEvent(graphene.Interface):
     label = graphene.NonNull(graphene.String)
     description = graphene.String()
-    metadataEntries = non_null_list(GrapheneEventMetadataEntry)
+    metadataEntries = non_null_list(GrapheneMetadataEntry)
 
     class Meta:
         name = "DisplayableEvent"
@@ -55,69 +50,73 @@ class GrapheneLogMessageEvent(graphene.ObjectType):
         name = "LogMessageEvent"
 
 
-class GraphenePipelineEvent(graphene.Interface):
+class GrapheneRunEvent(graphene.Interface):
     pipelineName = graphene.NonNull(graphene.String)
 
     class Meta:
-        name = "PipelineEvent"
+        name = "RunEvent"
 
 
-class GraphenePipelineEnqueuedEvent(graphene.ObjectType):
+class GrapheneRunEnqueuedEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineEnqueuedEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunEnqueuedEvent"
 
 
-class GraphenePipelineDequeuedEvent(graphene.ObjectType):
+class GrapheneRunDequeuedEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineDequeuedEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunDequeuedEvent"
 
 
-class GraphenePipelineStartingEvent(graphene.ObjectType):
+class GrapheneRunStartingEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineStartingEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunStartingEvent"
 
 
-class GraphenePipelineCancelingEvent(graphene.ObjectType):
+class GrapheneRunCancelingEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineCancelingEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunCancelingEvent"
 
 
-class GraphenePipelineCanceledEvent(graphene.ObjectType):
+class GrapheneRunCanceledEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineCanceledEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunCanceledEvent"
 
 
-class GraphenePipelineStartEvent(graphene.ObjectType):
+class GrapheneRunStartEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineStartEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunStartEvent"
 
 
-class GraphenePipelineSuccessEvent(graphene.ObjectType):
+class GrapheneRunSuccessEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineSuccessEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunSuccessEvent"
 
 
-class GraphenePipelineFailureEvent(graphene.ObjectType):
+class GrapheneRunFailureEvent(graphene.ObjectType):
     error = graphene.Field(GraphenePythonError)
 
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineFailureEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "RunFailureEvent"
 
 
-class GraphenePipelineInitFailureEvent(graphene.ObjectType):
-    error = graphene.NonNull(GraphenePythonError)
-
+class GrapheneAlertStartEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GraphenePipelineEvent)
-        name = "PipelineInitFailureEvent"
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "AlertStartEvent"
+
+
+class GrapheneAlertSuccessEvent(graphene.ObjectType):
+    class Meta:
+        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
+        name = "AlertSuccessEvent"
 
 
 class GrapheneStepEvent(graphene.Interface):
@@ -141,7 +140,7 @@ class GrapheneExecutionStepRestartEvent(graphene.ObjectType):
 
 
 class GrapheneExecutionStepUpForRetryEvent(graphene.ObjectType):
-    error = graphene.NonNull(GraphenePythonError)
+    error = graphene.Field(GraphenePythonError)
     secondsToWait = graphene.Field(graphene.Int)
 
     class Meta:
@@ -155,71 +154,6 @@ class GrapheneExecutionStepSkippedEvent(graphene.ObjectType):
         name = "ExecutionStepSkippedEvent"
 
 
-class GrapheneEventPathMetadataEntry(graphene.ObjectType):
-    path = graphene.NonNull(graphene.String)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventPathMetadataEntry"
-
-
-class GrapheneEventJsonMetadataEntry(graphene.ObjectType):
-    jsonString = graphene.NonNull(graphene.String)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventJsonMetadataEntry"
-
-
-class GrapheneEventTextMetadataEntry(graphene.ObjectType):
-    text = graphene.NonNull(graphene.String)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventTextMetadataEntry"
-
-
-class GrapheneEventUrlMetadataEntry(graphene.ObjectType):
-    url = graphene.NonNull(graphene.String)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventUrlMetadataEntry"
-
-
-class GrapheneEventMarkdownMetadataEntry(graphene.ObjectType):
-    md_str = graphene.NonNull(graphene.String)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventMarkdownMetadataEntry"
-
-
-class GrapheneEventPythonArtifactMetadataEntry(graphene.ObjectType):
-    module = graphene.NonNull(graphene.String)
-    name = graphene.NonNull(graphene.String)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventPythonArtifactMetadataEntry"
-
-
-class GrapheneEventFloatMetadataEntry(graphene.ObjectType):
-    floatValue = graphene.NonNull(graphene.Float)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventFloatMetadataEntry"
-
-
-class GrapheneEventIntMetadataEntry(graphene.ObjectType):
-    intValue = graphene.NonNull(graphene.Int)
-
-    class Meta:
-        interfaces = (GrapheneEventMetadataEntry,)
-        name = "EventIntMetadataEntry"
-
-
 class GrapheneObjectStoreOperationType(graphene.Enum):
     SET_OBJECT = "SET_OBJECT"
     GET_OBJECT = "GET_OBJECT"
@@ -243,27 +177,6 @@ class GrapheneObjectStoreOperationResult(graphene.ObjectType):
         return _to_metadata_entries(self.metadata_entries)  # pylint: disable=no-member
 
 
-class GrapheneMaterialization(graphene.ObjectType):
-    assetKey = graphene.Field(GrapheneAssetKey)
-
-    class Meta:
-        interfaces = (GrapheneDisplayableEvent,)
-        name = "Materialization"
-
-    def resolve_metadataEntries(self, _graphene_info):
-        from ...implementation.events import _to_metadata_entries
-
-        return _to_metadata_entries(self.metadata_entries)  # pylint: disable=no-member
-
-    def resolve_assetKey(self, _graphene_info):
-        asset_key = self.asset_key  # pylint: disable=no-member
-
-        if not asset_key:
-            return None
-
-        return GrapheneAssetKey(path=asset_key.path)
-
-
 class GrapheneExpectationResult(graphene.ObjectType):
     success = graphene.NonNull(graphene.Boolean)
 
@@ -330,7 +243,7 @@ class GrapheneExecutionStepFailureEvent(graphene.ObjectType):
         interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
         name = "ExecutionStepFailureEvent"
 
-    error = graphene.NonNull(GraphenePythonError)
+    error = graphene.Field(GraphenePythonError)
     errorSource = graphene.Field(graphene.Enum.from_enum(ErrorSource))
     failureMetadata = graphene.Field(GrapheneFailureMetadata)
 
@@ -352,21 +265,45 @@ class GrapheneHookErroredEvent(graphene.ObjectType):
         interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
         name = "HookErroredEvent"
 
-    error = graphene.NonNull(GraphenePythonError)
+    error = graphene.Field(GraphenePythonError)
 
 
-class GrapheneStepMaterializationEvent(graphene.ObjectType):
+class GrapheneLogsCapturedEvent(graphene.ObjectType):
     class Meta:
-        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
-        name = "StepMaterializationEvent"
+        interfaces = (GrapheneMessageEvent,)
+        name = "LogsCapturedEvent"
 
-    materialization = graphene.NonNull(GrapheneMaterialization)
-    stepStats = graphene.NonNull(lambda: GraphenePipelineRunStepStats)
-    assetLineage = non_null_list(GrapheneAssetLineageInfo)
+    logKey = graphene.NonNull(graphene.String)
+    stepKeys = graphene.List(graphene.NonNull(graphene.String))
+    pid = graphene.Int()
 
-    def __init__(self, materialization, assetLineage, **basic_params):
-        self._asset_lineage = check.opt_list_param(assetLineage, "assetLineage", AssetLineageInfo)
-        super().__init__(materialization=materialization, **basic_params)
+
+def _construct_asset_event_metadata_params(event, metadata):
+    metadata_params = {"label": metadata.label, "description": metadata.description}
+    metadata_params.update(construct_basic_params(event))
+    return metadata_params
+
+
+class AssetEventMixin:
+    assetKey = graphene.Field(GrapheneAssetKey)
+    runOrError = graphene.NonNull("dagster_graphql.schema.pipelines.pipeline.GrapheneRunOrError")
+    stepStats = graphene.NonNull(lambda: GrapheneRunStepStats)
+    partition = graphene.Field(graphene.String)
+
+    def __init__(self, event, metadata):
+        self._event = event
+        self._metadata = metadata
+
+    def resolve_assetKey(self, _graphene_info):
+        asset_key = self._metadata.asset_key
+
+        if not asset_key:
+            return None
+
+        return GrapheneAssetKey(path=asset_key.path)
+
+    def resolve_runOrError(self, graphene_info):
+        return get_run_by_id(graphene_info, self._event.run_id)
 
     def resolve_stepStats(self, graphene_info):
         run_id = self.runId  # pylint: disable=no-member
@@ -374,6 +311,46 @@ class GrapheneStepMaterializationEvent(graphene.ObjectType):
         stats = get_step_stats(graphene_info, run_id, step_keys=[step_key])
         return stats[0]
 
+    def resolve_metadataEntries(self, _graphene_info):
+        from ...implementation.events import _to_metadata_entries
+
+        return _to_metadata_entries(self._metadata.metadata_entries)
+
+    def resolve_partition(self, _graphene_info):
+        return self._metadata.partition
+
+
+class GrapheneMaterializationEvent(graphene.ObjectType, AssetEventMixin):
+    class Meta:
+        interfaces = (GrapheneMessageEvent, GrapheneStepEvent, GrapheneDisplayableEvent)
+        name = "MaterializationEvent"
+
+    assetLineage = non_null_list(GrapheneAssetLineageInfo)
+
+    def __init__(self, event, assetLineage=None, loader=None):
+        self._asset_lineage = check.opt_list_param(assetLineage, "assetLineage", AssetLineageInfo)
+        self._batch_run_loader = check.opt_inst_param(loader, "loader", BatchRunLoader)
+
+        materialization = event.dagster_event.step_materialization_data.materialization
+        super().__init__(**_construct_asset_event_metadata_params(event, materialization))
+        AssetEventMixin.__init__(
+            self,
+            event=event,
+            metadata=materialization,
+        )
+
+    def resolve_runOrError(self, graphene_info):
+        from ..pipelines.pipeline import GrapheneRun
+
+        if self._batch_run_loader:
+            record = self._batch_run_loader.get_run_record_by_run_id(self._event.run_id)
+            if not record:
+                return GrapheneRunNotFoundError(self._event.run_id)
+
+            return GrapheneRun(record)
+
+        return super().resolve_runOrError(graphene_info)
+
     def resolve_assetLineage(self, _graphene_info):
         return [
             GrapheneAssetLineageInfo(
@@ -384,6 +361,21 @@ class GrapheneStepMaterializationEvent(graphene.ObjectType):
         ]
 
 
+class GrapheneObservationEvent(graphene.ObjectType, AssetEventMixin):
+    class Meta:
+        interfaces = (GrapheneMessageEvent, GrapheneStepEvent, GrapheneDisplayableEvent)
+        name = "ObservationEvent"
+
+    def __init__(self, event):
+        observation = event.dagster_event.asset_observation_data.asset_observation
+        super().__init__(**_construct_asset_event_metadata_params(event, observation))
+        AssetEventMixin.__init__(
+            self,
+            event=event,
+            metadata=observation,
+        )
+
+
 class GrapheneHandledOutputEvent(graphene.ObjectType):
     class Meta:
         interfaces = (GrapheneMessageEvent, GrapheneStepEvent, GrapheneDisplayableEvent)
@@ -431,7 +423,7 @@ class GrapheneStepExpectationResultEvent(graphene.ObjectType):
 
 
 # Should be a union of all possible events
-class GraphenePipelineRunEvent(graphene.Union):
+class GrapheneDagsterRunEvent(graphene.Union):
     class Meta:
         types = (
             GrapheneExecutionStepFailureEvent,
@@ -443,40 +435,67 @@ class GraphenePipelineRunEvent(graphene.Union):
             GrapheneExecutionStepUpForRetryEvent,
             GrapheneExecutionStepRestartEvent,
             GrapheneLogMessageEvent,
-            GraphenePipelineFailureEvent,
-            GraphenePipelineInitFailureEvent,
-            GraphenePipelineStartEvent,
-            GraphenePipelineEnqueuedEvent,
-            GraphenePipelineDequeuedEvent,
-            GraphenePipelineStartingEvent,
-            GraphenePipelineCancelingEvent,
-            GraphenePipelineCanceledEvent,
-            GraphenePipelineSuccessEvent,
+            GrapheneRunFailureEvent,
+            GrapheneRunStartEvent,
+            GrapheneRunEnqueuedEvent,
+            GrapheneRunDequeuedEvent,
+            GrapheneRunStartingEvent,
+            GrapheneRunCancelingEvent,
+            GrapheneRunCanceledEvent,
+            GrapheneRunSuccessEvent,
             GrapheneHandledOutputEvent,
             GrapheneLoadedInputEvent,
+            GrapheneLogsCapturedEvent,
             GrapheneObjectStoreOperationEvent,
             GrapheneStepExpectationResultEvent,
-            GrapheneStepMaterializationEvent,
+            GrapheneMaterializationEvent,
+            GrapheneObservationEvent,
             GrapheneEngineEvent,
             GrapheneHookCompletedEvent,
             GrapheneHookSkippedEvent,
             GrapheneHookErroredEvent,
+            GrapheneAlertStartEvent,
+            GrapheneAlertSuccessEvent,
         )
-        name = "PipelineRunEvent"
+        name = "DagsterRunEvent"
 
 
-class GraphenePipelineRunStepStats(graphene.ObjectType):
+class GraphenePipelineRunStepStats(graphene.Interface):
     runId = graphene.NonNull(graphene.String)
     stepKey = graphene.NonNull(graphene.String)
     status = graphene.Field(GrapheneStepEventStatus)
     startTime = graphene.Field(graphene.Float)
     endTime = graphene.Field(graphene.Float)
-    materializations = non_null_list(GrapheneMaterialization)
+    materializations = non_null_list(GrapheneMaterializationEvent)
     expectationResults = non_null_list(GrapheneExpectationResult)
 
     class Meta:
         name = "PipelineRunStepStats"
 
+
+class GrapheneRunMarker(graphene.ObjectType):
+    startTime = graphene.Field(graphene.Float)
+    endTime = graphene.Field(graphene.Float)
+
+    class Meta:
+        name = "RunMarker"
+
+
+class GrapheneRunStepStats(graphene.ObjectType):
+    runId = graphene.NonNull(graphene.String)
+    stepKey = graphene.NonNull(graphene.String)
+    status = graphene.Field(GrapheneStepEventStatus)
+    startTime = graphene.Field(graphene.Float)
+    endTime = graphene.Field(graphene.Float)
+    materializations = non_null_list(GrapheneMaterializationEvent)
+    expectationResults = non_null_list(GrapheneExpectationResult)
+    attempts = non_null_list(GrapheneRunMarker)
+    markers = non_null_list(GrapheneRunMarker)
+
+    class Meta:
+        interfaces = (GraphenePipelineRunStepStats,)
+        name = "RunStepStats"
+
     def __init__(self, stats):
         self._stats = check.inst_param(stats, "stats", RunStepKeyStatsSnapshot)
         super().__init__(
@@ -485,6 +504,14 @@ class GraphenePipelineRunStepStats(graphene.ObjectType):
             status=stats.status,
             startTime=stats.start_time,
             endTime=stats.end_time,
-            materializations=stats.materializations,
+            materializations=stats.materialization_events,
             expectationResults=stats.expectation_results,
+            attempts=[
+                GrapheneRunMarker(startTime=attempt.start_time, endTime=attempt.end_time)
+                for attempt in stats.attempts_list
+            ],
+            markers=[
+                GrapheneRunMarker(startTime=marker.start_time, endTime=marker.end_time)
+                for marker in stats.markers
+            ],
         )
