@@ -1,6 +1,7 @@
 import graphene
 from dagster import check
 from dagster.core.definitions.events import AssetKey
+from dagster.core.execution.backfill import BulkActionStatus
 from dagster.core.host_representation import (
     InstigationSelector,
     RepositorySelector,
@@ -12,7 +13,10 @@ from dagster.core.scheduler.instigation import InstigatorType
 from ...implementation.external import fetch_repositories, fetch_repository, fetch_workspace
 from ...implementation.fetch_assets import get_asset, get_asset_node, get_asset_nodes, get_assets
 from ...implementation.fetch_backfills import get_backfill, get_backfills
-from ...implementation.fetch_jobs import get_job_state_or_error, get_unloadable_job_states_or_error
+from ...implementation.fetch_instigators import (
+    get_instigator_state_or_error,
+    get_unloadable_instigator_states_or_error,
+)
 from ...implementation.fetch_partition_sets import get_partition_set, get_partition_sets_or_error
 from ...implementation.fetch_pipelines import (
     get_pipeline_or_error,
@@ -34,10 +38,15 @@ from ...implementation.fetch_schedules import (
 )
 from ...implementation.fetch_sensors import get_sensor_or_error, get_sensors_or_error
 from ...implementation.fetch_solids import get_graph_or_error
+from ...implementation.loader import BatchMaterializationLoader
 from ...implementation.run_config_schema import resolve_run_config_schema_or_error
 from ...implementation.utils import graph_selector_from_graphql, pipeline_selector_from_graphql
 from ..asset_graph import GrapheneAssetNode, GrapheneAssetNodeOrError
-from ..backfill import GraphenePartitionBackfillOrError, GraphenePartitionBackfillsOrError
+from ..backfill import (
+    GrapheneBulkActionStatus,
+    GraphenePartitionBackfillOrError,
+    GraphenePartitionBackfillsOrError,
+)
 from ..external import (
     GrapheneRepositoriesOrError,
     GrapheneRepositoryOrError,
@@ -71,6 +80,7 @@ from ..runs import (
     GrapheneRunGroupsOrError,
     GrapheneRuns,
     GrapheneRunsOrError,
+    parse_run_config_input,
 )
 from ..schedules import GrapheneScheduleOrError, GrapheneSchedulerOrError, GrapheneSchedulesOrError
 from ..sensors import GrapheneSensorOrError, GrapheneSensorsOrError
@@ -225,7 +235,12 @@ class GrapheneDagitQuery(graphene.ObjectType):
         assetKey=graphene.Argument(graphene.NonNull(GrapheneAssetKeyInput)),
     )
 
-    assetNodes = non_null_list(GrapheneAssetNode)
+    assetNodes = graphene.Field(
+        non_null_list(GrapheneAssetNode),
+        pipeline=graphene.Argument(GraphenePipelineSelector),
+        assetKeys=graphene.Argument(graphene.List(graphene.NonNull(GrapheneAssetKeyInput))),
+        loadMaterializations=graphene.Boolean(default_value=False),
+    )
 
     assetNodeOrError = graphene.Field(
         graphene.NonNull(GrapheneAssetNodeOrError),
@@ -239,6 +254,7 @@ class GrapheneDagitQuery(graphene.ObjectType):
 
     partitionBackfillsOrError = graphene.Field(
         graphene.NonNull(GraphenePartitionBackfillsOrError),
+        status=graphene.Argument(GrapheneBulkActionStatus),
         cursor=graphene.String(),
         limit=graphene.Int(),
     )
@@ -308,15 +324,15 @@ class GrapheneDagitQuery(graphene.ObjectType):
         )
 
     def resolve_instigationStateOrError(self, graphene_info, instigationSelector):
-        return get_job_state_or_error(
+        return get_instigator_state_or_error(
             graphene_info, InstigationSelector.from_graphql_input(instigationSelector)
         )
 
     def resolve_unloadableInstigationStatesOrError(self, graphene_info, **kwargs):
-        job_type = (
+        instigation_type = (
             InstigatorType(kwargs["instigationType"]) if "instigationType" in kwargs else None
         )
-        return get_unloadable_job_states_or_error(graphene_info, job_type)
+        return get_unloadable_instigator_states_or_error(graphene_info, instigation_type)
 
     def resolve_pipelineOrError(self, graphene_info, **kwargs):
         return get_pipeline_or_error(
@@ -387,7 +403,7 @@ class GrapheneDagitQuery(graphene.ObjectType):
         return validate_pipeline_config(
             graphene_info,
             pipeline_selector_from_graphql(pipeline),
-            kwargs.get("runConfigData"),
+            parse_run_config_input(kwargs.get("runConfigData", {})),
             kwargs.get("mode"),
         )
 
@@ -395,7 +411,7 @@ class GrapheneDagitQuery(graphene.ObjectType):
         return get_execution_plan(
             graphene_info,
             pipeline_selector_from_graphql(pipeline),
-            kwargs.get("runConfigData"),
+            parse_run_config_input(kwargs.get("runConfigData", {})),
             kwargs.get("mode"),
         )
 
@@ -409,8 +425,46 @@ class GrapheneDagitQuery(graphene.ObjectType):
     def resolve_instance(self, graphene_info):
         return GrapheneInstance(graphene_info.context.instance)
 
-    def resolve_assetNodes(self, graphene_info):
-        return get_asset_nodes(graphene_info)
+    def resolve_assetNodes(self, graphene_info, **kwargs):
+        asset_keys = set(
+            AssetKey.from_graphql_input(asset_key) for asset_key in kwargs.get("assetKeys", [])
+        )
+
+        if "pipeline" in kwargs:
+            pipeline_name = kwargs.get("pipeline").get("pipelineName")
+            repo_sel = RepositorySelector.from_graphql_input(kwargs.get("pipeline"))
+            repo_loc = graphene_info.context.get_repository_location(repo_sel.location_name)
+            repo = repo_loc.get_repository(repo_sel.repository_name)
+            external_asset_nodes = repo.get_external_asset_nodes(pipeline_name)
+            results = (
+                [
+                    GrapheneAssetNode(repo_loc, repo, asset_node)
+                    for asset_node in external_asset_nodes
+                ]
+                if external_asset_nodes
+                else []
+            )
+        else:
+            results = get_asset_nodes(graphene_info)
+
+        # Filter down to requested asset keys
+        results = [node for node in results if not asset_keys or node.assetKey in asset_keys]
+
+        if not results:
+            return []
+
+        materialization_loader = BatchMaterializationLoader(
+            instance=graphene_info.context.instance, asset_keys=[node.assetKey for node in results]
+        )
+        return [
+            GrapheneAssetNode(
+                node.repository_location,
+                node.external_repository,
+                node.get_external_asset_node(),
+                materialization_loader=materialization_loader,
+            )
+            for node in results
+        ]
 
     def resolve_assetNodeOrError(self, graphene_info, **kwargs):
         return get_asset_node(graphene_info, AssetKey.from_graphql_input(kwargs["assetKey"]))
@@ -430,8 +484,10 @@ class GrapheneDagitQuery(graphene.ObjectType):
         return get_backfill(graphene_info, backfillId)
 
     def resolve_partitionBackfillsOrError(self, graphene_info, **kwargs):
+        status = kwargs.get("status")
         return get_backfills(
             graphene_info,
+            status=BulkActionStatus.from_graphql_input(status) if status else None,
             cursor=kwargs.get("cursor"),
             limit=kwargs.get("limit"),
         )
