@@ -1,8 +1,8 @@
 import graphene
 from dagster import check
 from dagster.core.host_representation import ExternalSchedule
-from dagster.core.scheduler.instigation import InstigatorState
 from dagster.seven import get_current_datetime_in_utc, get_timestamp_from_utc_datetime
+from dagster_graphql.implementation.loader import RepositoryScopedBatchLoader
 
 from ..errors import (
     GraphenePythonError,
@@ -29,7 +29,10 @@ class GrapheneSchedule(graphene.ObjectType):
     scheduleState = graphene.NonNull(GrapheneInstigationState)
     partition_set = graphene.Field("dagster_graphql.schema.partition_sets.GraphenePartitionSet")
     futureTicks = graphene.NonNull(
-        GrapheneFutureInstigationTicks, cursor=graphene.Float(), limit=graphene.Int()
+        GrapheneFutureInstigationTicks,
+        cursor=graphene.Float(),
+        limit=graphene.Int(),
+        until=graphene.Float(),
     )
     futureTick = graphene.NonNull(
         GrapheneFutureInstigationTick, tick_timestamp=graphene.NonNull(graphene.Int)
@@ -38,15 +41,18 @@ class GrapheneSchedule(graphene.ObjectType):
     class Meta:
         name = "Schedule"
 
-    def __init__(self, external_schedule, schedule_state):
+    def __init__(self, external_schedule, schedule_state, batch_loader=None):
         self._external_schedule = check.inst_param(
             external_schedule, "external_schedule", ExternalSchedule
         )
-        self._schedule_state = check.opt_inst_param(
-            schedule_state, "schedule_state", InstigatorState
+
+        # optional run loader, provided by a parent graphene object (e.g. GrapheneRepository)
+        # that instantiates multiple schedules
+        self._batch_loader = check.opt_inst_param(
+            batch_loader, "batch_loader", RepositoryScopedBatchLoader
         )
-        if not self._schedule_state:
-            self._schedule_state = external_schedule.get_default_instigation_state()
+
+        self._schedule_state = self._external_schedule.get_current_instigator_state(schedule_state)
 
         super().__init__(
             name=external_schedule.name,
@@ -54,7 +60,6 @@ class GrapheneSchedule(graphene.ObjectType):
             pipeline_name=external_schedule.pipeline_name,
             solid_selection=external_schedule.solid_selection,
             mode=external_schedule.mode,
-            scheduleState=GrapheneInstigationState(self._schedule_state),
             execution_timezone=(
                 self._external_schedule.execution_timezone
                 if self._external_schedule.execution_timezone
@@ -66,6 +71,10 @@ class GrapheneSchedule(graphene.ObjectType):
     def resolve_id(self, _):
         return self._external_schedule.get_external_origin_id()
 
+    def resolve_scheduleState(self, _graphene_info):
+        # forward the batch run loader to the instigation state, which provides the schedule runs
+        return GrapheneInstigationState(self._schedule_state, self._batch_loader)
+
     def resolve_partition_set(self, graphene_info):
         from ..partition_sets import GraphenePartitionSet
 
@@ -88,13 +97,27 @@ class GrapheneSchedule(graphene.ObjectType):
         cursor = kwargs.get(
             "cursor", get_timestamp_from_utc_datetime(get_current_datetime_in_utc())
         )
-        limit = kwargs.get("limit", 10)
-
         tick_times = []
         time_iter = self._external_schedule.execution_time_iterator(cursor)
 
-        for _ in range(limit):
-            tick_times.append(next(time_iter).timestamp())
+        until = float(kwargs.get("until")) if kwargs.get("until") else None
+
+        if until:
+            currentTime = None
+            while (not currentTime or currentTime < until) and (
+                not kwargs.get("limit") or len(tick_times) < kwargs.get("limit")
+            ):
+                try:
+                    currentTime = next(time_iter).timestamp()
+                    if currentTime < until:
+                        tick_times.append(currentTime)
+                except StopIteration:
+                    break
+        else:
+            limit = kwargs.get("limit", 10)
+
+            for _ in range(limit):
+                tick_times.append(next(time_iter).timestamp())
 
         future_ticks = [
             GrapheneFutureInstigationTick(self._schedule_state, tick_time)
