@@ -4,10 +4,16 @@ import graphene
 from dagster import check
 from dagster.core.definitions import NodeHandle
 from dagster.core.host_representation import RepresentedPipeline
+from dagster.core.host_representation.historical import HistoricalPipeline
 from dagster.core.snap import CompositeSolidDefSnap, DependencyStructureIndex, SolidDefSnap
+from dagster.core.storage.pipeline_run import PipelineRunsFilter
+from dagster_graphql.implementation.events import iterate_metadata_entries
+from dagster_graphql.schema.logs.events import GrapheneRunStepStats
+from dagster_graphql.schema.metadata import GrapheneMetadataEntry
 
 from .config_types import GrapheneConfigTypeField
 from .dagster_types import GrapheneDagsterType, to_dagster_type
+from .errors import GrapheneError
 from .metadata import GrapheneMetadataItemDefinition
 from .util import non_null_list
 
@@ -21,6 +27,7 @@ class GrapheneInputDefinition(graphene.ObjectType):
     name = graphene.NonNull(graphene.String)
     description = graphene.String()
     type = graphene.NonNull(GrapheneDagsterType)
+    metadata_entries = non_null_list(GrapheneMetadataEntry)
 
     class Meta:
         name = "InputDefinition"
@@ -48,6 +55,9 @@ class GrapheneInputDefinition(graphene.ObjectType):
             self._represented_pipeline, self._solid_def_snap.name  # pylint: disable=no-member
         )
 
+    def resolve_metadata_entries(self, _graphene_info):
+        return list(iterate_metadata_entries(self._input_def_snap.metadata_entries))
+
 
 class GrapheneOutputDefinition(graphene.ObjectType):
     solid_definition = graphene.NonNull(lambda: GrapheneSolidDefinition)
@@ -55,6 +65,7 @@ class GrapheneOutputDefinition(graphene.ObjectType):
     description = graphene.String()
     is_dynamic = graphene.Boolean()
     type = graphene.NonNull(GrapheneDagsterType)
+    metadata_entries = non_null_list(GrapheneMetadataEntry)
 
     class Meta:
         name = "OutputDefinition"
@@ -84,6 +95,9 @@ class GrapheneOutputDefinition(graphene.ObjectType):
     def resolve_solid_definition(self, _graphene_info):
         return build_solid_definition(self._represented_pipeline, self._solid_def_snap.name)
 
+    def resolve_metadata_entries(self, _graphene_info):
+        return list(iterate_metadata_entries(self._output_def_snap.metadata_entries))
+
 
 class GrapheneInput(graphene.ObjectType):
     solid = graphene.NonNull(lambda: GrapheneSolid)
@@ -337,6 +351,7 @@ class GrapheneISolidDefinition(graphene.Interface):
     metadata = non_null_list(GrapheneMetadataItemDefinition)
     input_definitions = non_null_list(GrapheneInputDefinition)
     output_definitions = non_null_list(GrapheneOutputDefinition)
+    asset_nodes = non_null_list("dagster_graphql.schema.asset_graph.GrapheneAssetNode")
 
     class Meta:
         name = "ISolidDefinition"
@@ -379,6 +394,27 @@ class ISolidDefinitionMixin:
             for output_def_snap in self._solid_def_snap.output_def_snaps
         ]
 
+    def resolve_asset_nodes(self, graphene_info):
+        # NOTE: This is a temporary hack. We really should prob be resolving solids against the repo
+        # rather than pipeline, that way we would not have to refetch the repo here here in order to
+        # access the asset nodes.
+        from .asset_graph import GrapheneAssetNode
+
+        # This is a workaround for the fact that asset info is not persisted in pipeline snapshots.
+        if isinstance(self._represented_pipeline, HistoricalPipeline):
+            return []
+        else:
+            repo_handle = self._represented_pipeline.repository_handle
+            origin = repo_handle.repository_location_origin
+            location = graphene_info.context.get_location(origin)
+            ext_repo = location.get_repository(repo_handle.repository_name)
+            nodes = [
+                node
+                for node in ext_repo.get_external_asset_nodes()
+                if node.op_name == self.solid_def_name
+            ]
+            return [GrapheneAssetNode(location, ext_repo, node) for node in nodes]
+
 
 class GrapheneSolidDefinition(graphene.ObjectType, ISolidDefinitionMixin):
     config_field = graphene.Field(GrapheneConfigTypeField)
@@ -410,11 +446,31 @@ class GrapheneSolidDefinition(graphene.ObjectType, ISolidDefinitionMixin):
         ]
 
 
+class GrapheneSolidStepStatsUnavailableError(graphene.ObjectType):
+    class Meta:
+        interfaces = (GrapheneError,)
+        name = "SolidStepStatusUnavailableError"
+
+
+class GrapheneSolidStepStatsConnection(graphene.ObjectType):
+    nodes = non_null_list(GrapheneRunStepStats)
+
+    class Meta:
+        name = "SolidStepStatsConnection"
+
+
+class GrapheneSolidStepStatsOrError(graphene.Union):
+    class Meta:
+        types = (GrapheneSolidStepStatsConnection, GrapheneSolidStepStatsUnavailableError)
+        name = "SolidStepStatsOrError"
+
+
 class GrapheneSolid(graphene.ObjectType):
     name = graphene.NonNull(graphene.String)
     definition = graphene.NonNull(lambda: GrapheneISolidDefinition)
     inputs = non_null_list(GrapheneInput)
     outputs = non_null_list(GrapheneOutput)
+
     is_dynamic_mapped = graphene.NonNull(graphene.Boolean)
 
     class Meta:
@@ -441,6 +497,15 @@ class GrapheneSolid(graphene.ObjectType):
     def get_solid_definition(self):
         return build_solid_definition(self._represented_pipeline, self._solid_def_snap.name)
 
+    def get_is_dynamic_mapped(self):
+        return self._solid_invocation_snap.is_dynamic_mapped
+
+    def get_is_composite(self):
+        return isinstance(self._solid_def_snap, CompositeSolidDefSnap)
+
+    def get_pipeline_name(self):
+        return self._represented_pipeline.name
+
     def resolve_definition(self, _graphene_info):
         return self.get_solid_definition()
 
@@ -474,6 +539,7 @@ class GrapheneSolidHandle(graphene.ObjectType):
     handleID = graphene.NonNull(graphene.String)
     solid = graphene.NonNull(GrapheneSolid)
     parent = graphene.Field(lambda: GrapheneSolidHandle)
+    stepStats = graphene.Field(GrapheneSolidStepStatsOrError, limit=graphene.Int())
 
     class Meta:
         name = "SolidHandle"
@@ -484,6 +550,28 @@ class GrapheneSolidHandle(graphene.ObjectType):
             solid=check.inst_param(solid, "solid", GrapheneSolid),
             parent=check.opt_inst_param(parent, "parent", GrapheneSolidHandle),
         )
+        self._solid = solid
+
+    def resolve_stepStats(self, _graphene_info, limit):
+        if self._solid.get_is_dynamic_mapped():
+            return GrapheneSolidStepStatsUnavailableError(
+                message="Step stats are not available for dynamically-mapped ops"
+            )
+
+        if self._solid.get_is_composite():
+            return GrapheneSolidStepStatsUnavailableError(
+                message="Step stats are not available for composite solids / subgraphs"
+            )
+
+        instance = _graphene_info.context.instance
+        runs_filter = PipelineRunsFilter(pipeline_name=self._solid.get_pipeline_name())
+        runs = instance.get_runs(runs_filter, limit=limit)
+        nodes = []
+        for run in runs:
+            stats = instance.get_run_step_stats(run.run_id, [str(self.handleID)])
+            if len(stats):
+                nodes.append(GrapheneRunStepStats(stats[0]))
+        return GrapheneSolidStepStatsConnection(nodes=nodes)
 
 
 class GrapheneSolidContainer(graphene.Interface):
