@@ -1,16 +1,9 @@
-import sys
 from contextlib import contextmanager
 
 from dagster import check
-from dagster.cli.workspace import Workspace
-from dagster.core.definitions.reconstructable import ReconstructableRepository
-from dagster.core.host_representation import (
-    InProcessRepositoryLocationOrigin,
-    ManagedGrpcPythonEnvRepositoryLocationOrigin,
-)
 from dagster.core.instance import DagsterInstance
-from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
-from dagster_graphql.implementation.context import DagsterGraphQLContext
+from dagster.core.workspace import WorkspaceProcessContext
+from dagster.core.workspace.load_target import PythonFileTarget
 from dagster_graphql.schema import create_schema
 from graphql import graphql
 
@@ -50,38 +43,27 @@ def execute_dagster_graphql_and_finish_runs(context, query, variables=None):
     return result
 
 
-def define_in_process_context(python_file, fn_name, instance):
-    check.inst_param(instance, "instance", DagsterInstance)
-
-    return DagsterGraphQLContext(
-        workspace=Workspace(
-            [
-                InProcessRepositoryLocationOrigin(
-                    ReconstructableRepository.for_file(python_file, fn_name)
-                )
-            ]
-        ),
-        instance=instance,
-    )
-
-
 @contextmanager
 def define_out_of_process_context(python_file, fn_name, instance):
     check.inst_param(instance, "instance", DagsterInstance)
 
-    with Workspace(
-        [
-            ManagedGrpcPythonEnvRepositoryLocationOrigin(
-                loadable_target_origin=LoadableTargetOrigin(
-                    executable_path=sys.executable, python_file=python_file, attribute=fn_name,
-                ),
-                location_name=main_repo_location_name(),
-            )
-        ]
-    ) as workspace:
-        yield DagsterGraphQLContext(
-            workspace=workspace, instance=instance,
-        )
+    with define_out_of_process_workspace(
+        python_file, fn_name, instance
+    ) as workspace_process_context:
+        yield workspace_process_context.create_request_context()
+
+
+def define_out_of_process_workspace(python_file, fn_name, instance):
+    return WorkspaceProcessContext(
+        instance,
+        PythonFileTarget(
+            python_file=python_file,
+            attribute=fn_name,
+            working_directory=None,
+            location_name=main_repo_location_name(),
+        ),
+        version="",
+    )
 
 
 def infer_repository(graphql_context):
@@ -131,7 +113,7 @@ def infer_sensor_selector(graphql_context, sensor_name):
     return selector
 
 
-def infer_job_selector(graphql_context, job_name):
+def infer_instigation_selector(graphql_context, name):
     selector = infer_repository_selector(graphql_context)
-    selector.update({"jobName": job_name})
+    selector.update({"name": name})
     return selector
