@@ -17,6 +17,27 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[asset_aware_instance_in_pr
                     ]
                 }
             },
+            {
+                'key': {
+                    'path': [
+                        'asset_1'
+                    ]
+                }
+            },
+            {
+                'key': {
+                    'path': [
+                        'asset_2'
+                    ]
+                }
+            },
+            {
+                'key': {
+                    'path': [
+                        'asset_3'
+                    ]
+                }
+            },
             {
                 'key': {
                     'path': [
@@ -31,6 +52,13 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[asset_aware_instance_in_pr
                     ]
                 }
             },
+            {
+                'key': {
+                    'path': [
+                        'asset_yields_observation'
+                    ]
+                }
+            },
             {
                 'key': {
                     'path': [
@@ -62,7 +90,7 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[asset_aware_instance_in_pr
             {
                 'key': {
                     'path': [
-                        'dummy_foreign_asset'
+                        'dummy_source_asset'
                     ]
                 }
             },
@@ -100,6 +128,13 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[asset_aware_instance_in_pr
                         'upstream_time_partitioned_asset'
                     ]
                 }
+            },
+            {
+                'key': {
+                    'path': [
+                        'yield_partition_materialization'
+                    ]
+                }
             }
         ]
     }
@@ -116,6 +151,27 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[postgres_with_default_run_
                     ]
                 }
             },
+            {
+                'key': {
+                    'path': [
+                        'asset_1'
+                    ]
+                }
+            },
+            {
+                'key': {
+                    'path': [
+                        'asset_2'
+                    ]
+                }
+            },
+            {
+                'key': {
+                    'path': [
+                        'asset_3'
+                    ]
+                }
+            },
             {
                 'key': {
                     'path': [
@@ -130,6 +186,13 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[postgres_with_default_run_
                     ]
                 }
             },
+            {
+                'key': {
+                    'path': [
+                        'asset_yields_observation'
+                    ]
+                }
+            },
             {
                 'key': {
                     'path': [
@@ -161,7 +224,7 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[postgres_with_default_run_
             {
                 'key': {
                     'path': [
-                        'dummy_foreign_asset'
+                        'dummy_source_asset'
                     ]
                 }
             },
@@ -199,6 +262,13 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[postgres_with_default_run_
                         'upstream_time_partitioned_asset'
                     ]
                 }
+            },
+            {
+                'key': {
+                    'path': [
+                        'yield_partition_materialization'
+                    ]
+                }
             }
         ]
     }
@@ -215,6 +285,27 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[sqlite_with_default_run_la
                     ]
                 }
             },
+            {
+                'key': {
+                    'path': [
+                        'asset_1'
+                    ]
+                }
+            },
+            {
+                'key': {
+                    'path': [
+                        'asset_2'
+                    ]
+                }
+            },
+            {
+                'key': {
+                    'path': [
+                        'asset_3'
+                    ]
+                }
+            },
             {
                 'key': {
                     'path': [
@@ -229,6 +320,13 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[sqlite_with_default_run_la
                     ]
                 }
             },
+            {
+                'key': {
+                    'path': [
+                        'asset_yields_observation'
+                    ]
+                }
+            },
             {
                 'key': {
                     'path': [
@@ -260,7 +358,7 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[sqlite_with_default_run_la
             {
                 'key': {
                     'path': [
-                        'dummy_foreign_asset'
+                        'dummy_source_asset'
                     ]
                 }
             },
@@ -298,6 +396,13 @@ snapshots['TestAssetAwareEventLog.test_all_asset_keys[sqlite_with_default_run_la
                         'upstream_time_partitioned_asset'
                     ]
                 }
+            },
+            {
+                'key': {
+                    'path': [
+                        'yield_partition_materialization'
+                    ]
+                }
             }
         ]
     }
@@ -307,22 +412,9 @@ snapshots['TestAssetAwareEventLog.test_get_asset_key_lineage[asset_aware_instanc
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                        {
-                            'assetKey': {
-                                'path': [
-                                    'a'
-                                ]
-                            },
-                            'partitions': [
-                            ]
-                        }
-                    ],
-                    'materialization': {
-                        'label': 'b'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'b'
             }
         ]
     }
@@ -332,22 +424,9 @@ snapshots['TestAssetAwareEventLog.test_get_asset_key_lineage[postgres_with_defau
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                        {
-                            'assetKey': {
-                                'path': [
-                                    'a'
-                                ]
-                            },
-                            'partitions': [
-                            ]
-                        }
-                    ],
-                    'materialization': {
-                        'label': 'b'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'b'
             }
         ]
     }
@@ -357,22 +436,9 @@ snapshots['TestAssetAwareEventLog.test_get_asset_key_lineage[sqlite_with_default
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                        {
-                            'assetKey': {
-                                'path': [
-                                    'a'
-                                ]
-                            },
-                            'partitions': [
-                            ]
-                        }
-                    ],
-                    'materialization': {
-                        'label': 'b'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'b'
             }
         ]
     }
@@ -382,13 +448,9 @@ snapshots['TestAssetAwareEventLog.test_get_asset_key_materialization[asset_aware
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                    ],
-                    'materialization': {
-                        'label': 'a'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'a'
             }
         ]
     }
@@ -398,13 +460,9 @@ snapshots['TestAssetAwareEventLog.test_get_asset_key_materialization[postgres_wi
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                    ],
-                    'materialization': {
-                        'label': 'a'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'a'
             }
         ]
     }
@@ -414,13 +472,9 @@ snapshots['TestAssetAwareEventLog.test_get_asset_key_materialization[sqlite_with
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                    ],
-                    'materialization': {
-                        'label': 'a'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'a'
             }
         ]
     }
@@ -448,23 +502,9 @@ snapshots['TestAssetAwareEventLog.test_get_partitioned_asset_key_lineage[asset_a
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                        {
-                            'assetKey': {
-                                'path': [
-                                    'a'
-                                ]
-                            },
-                            'partitions': [
-                                '1'
-                            ]
-                        }
-                    ],
-                    'materialization': {
-                        'label': 'b'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'b'
             }
         ]
     }
@@ -474,23 +514,9 @@ snapshots['TestAssetAwareEventLog.test_get_partitioned_asset_key_lineage[postgre
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                        {
-                            'assetKey': {
-                                'path': [
-                                    'a'
-                                ]
-                            },
-                            'partitions': [
-                                '1'
-                            ]
-                        }
-                    ],
-                    'materialization': {
-                        'label': 'b'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'b'
             }
         ]
     }
@@ -500,23 +526,9 @@ snapshots['TestAssetAwareEventLog.test_get_partitioned_asset_key_lineage[sqlite_
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'assetLineage': [
-                        {
-                            'assetKey': {
-                                'path': [
-                                    'a'
-                                ]
-                            },
-                            'partitions': [
-                                '1'
-                            ]
-                        }
-                    ],
-                    'materialization': {
-                        'label': 'b'
-                    }
-                }
+                'assetLineage': [
+                ],
+                'label': 'b'
             }
         ]
     }
@@ -526,11 +538,7 @@ snapshots['TestAssetAwareEventLog.test_get_partitioned_asset_key_materialization
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'materialization': {
-                        'label': 'a'
-                    }
-                },
+                'label': 'a',
                 'partition': 'partition_1'
             }
         ]
@@ -541,11 +549,7 @@ snapshots['TestAssetAwareEventLog.test_get_partitioned_asset_key_materialization
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'materialization': {
-                        'label': 'a'
-                    }
-                },
+                'label': 'a',
                 'partition': 'partition_1'
             }
         ]
@@ -556,11 +560,7 @@ snapshots['TestAssetAwareEventLog.test_get_partitioned_asset_key_materialization
     'assetOrError': {
         'assetMaterializations': [
             {
-                'materializationEvent': {
-                    'materialization': {
-                        'label': 'a'
-                    }
-                },
+                'label': 'a',
                 'partition': 'partition_1'
             }
         ]
