@@ -13,5 +13,5 @@ def test_dagster_out_of_process_location():
             python_file=file_relative_path(__file__, "setup.py"),
             attribute="test_repo",
         ),
-    ).create_test_location() as env:
+    ).create_single_location() as env:
         assert env.get_repository("test_repo")
