@@ -31,10 +31,14 @@ query JobQuery($instigationSelector: InstigationSelector!) {
 """
 
 
-def _create_sensor_tick(instance):
-    with create_test_daemon_workspace() as workspace:
+def _create_sensor_tick(graphql_context):
+    with create_test_daemon_workspace(
+        graphql_context.process_context.workspace_load_target
+    ) as workspace:
         list(
-            execute_sensor_iteration(instance, get_default_daemon_logger("SensorDaemon"), workspace)
+            execute_sensor_iteration(
+                graphql_context.instance, get_default_daemon_logger("SensorDaemon"), workspace
+            )
         )
 
 
@@ -48,14 +52,13 @@ class TestNextTickRepository(
         external_repository = graphql_context.get_repository_location(
             repository_selector["repositoryLocationName"]
         ).get_repository(repository_selector["repositoryName"])
-        graphql_context.instance.reconcile_scheduler_state(external_repository)
 
         schedule_name = "no_config_pipeline_hourly_schedule"
         external_schedule = external_repository.get_external_schedule(schedule_name)
         selector = infer_instigation_selector(graphql_context, schedule_name)
 
         # need to be running in order to generate a future tick
-        graphql_context.instance.start_schedule_and_update_storage_state(external_schedule)
+        graphql_context.instance.start_schedule(external_schedule)
         result = execute_dagster_graphql(
             graphql_context, INSTIGATION_QUERY, variables={"instigationSelector": selector}
         )
@@ -70,7 +73,6 @@ class TestNextTickRepository(
         external_repository = graphql_context.get_repository_location(
             repository_selector["repositoryLocationName"]
         ).get_repository(repository_selector["repositoryName"])
-        graphql_context.instance.reconcile_scheduler_state(external_repository)
 
         sensor_name = "always_no_config_sensor"
         external_sensor = external_repository.get_external_sensor(sensor_name)
@@ -79,7 +81,7 @@ class TestNextTickRepository(
         # need to be running and create a sensor tick in the last 30 seconds in order to generate a
         # future tick
         graphql_context.instance.start_sensor(external_sensor)
-        _create_sensor_tick(graphql_context.instance)
+        _create_sensor_tick(graphql_context)
 
         result = execute_dagster_graphql(
             graphql_context, INSTIGATION_QUERY, variables={"instigationSelector": selector}
