@@ -13,6 +13,7 @@ from dagster.core.scheduler.instigation import (
     ScheduleInstigatorData,
 )
 from dagster.seven.compat.pendulum import create_pendulum_time
+from dagster.utils import Counter, traced_counter
 from dagster_graphql.test.utils import (
     execute_dagster_graphql,
     infer_repository_selector,
@@ -91,6 +92,20 @@ query getSchedule($scheduleSelector: ScheduleSelector!, $ticksAfter: Float) {
 }
 """
 
+GET_SCHEDULE_STATE_QUERY = """
+query getScheduleState($scheduleSelector: ScheduleSelector!) {
+  scheduleOrError(scheduleSelector: $scheduleSelector) {
+    __typename
+    ... on Schedule {
+      scheduleState {
+        id
+        status
+      }
+    }
+  }
+}
+"""
+
 GET_UNLOADABLE_QUERY = """
 query getUnloadableSchedules {
   unloadableInstigationStatesOrError(instigationType: SCHEDULE) {
@@ -155,6 +170,48 @@ mutation(
 }
 """
 
+GET_SCHEDULE_FUTURE_TICKS_UNTIL = """
+query getSchedule($scheduleSelector: ScheduleSelector!, $ticksAfter: Float, $ticksUntil: Float) {
+  scheduleOrError(scheduleSelector: $scheduleSelector) {
+    __typename
+    ... on PythonError {
+      message
+      stack
+    }
+    ... on Schedule {
+      name
+      futureTicks(cursor: $ticksAfter, until: $ticksUntil) {
+        results {
+          timestamp
+        }
+        cursor
+      }
+    }
+  }
+}
+"""
+
+REPOSITORY_SCHEDULES_QUERY = """
+query RepositorySchedulesQuery($repositorySelector: RepositorySelector!) {
+    repositoryOrError(repositorySelector: $repositorySelector) {
+        ... on Repository {
+            id
+            schedules {
+                id
+                name
+                scheduleState {
+                    id
+                    runs(limit: 1) {
+                      id
+                      runId
+                    }
+                }
+            }
+        }
+    }
+}
+"""
+
 
 def default_execution_params():
     return {
@@ -168,7 +225,7 @@ def _get_unloadable_schedule_origin(name):
     recon_repo = ReconstructableRepository.for_file(__file__, "doesnt_exist", working_directory)
     return ExternalRepositoryOrigin(
         InProcessRepositoryLocationOrigin(recon_repo), "fake_repository"
-    ).get_job_origin(name)
+    ).get_instigator_origin(name)
 
 
 def test_get_schedule_definitions_for_repository(graphql_context):
@@ -374,7 +431,7 @@ def test_get_unloadable_job(graphql_context):
         second=59,
     )
     with pendulum.test(initial_datetime):
-        instance.add_job_state(
+        instance.add_instigator_state(
             InstigatorState(
                 _get_unloadable_schedule_origin("unloadable_running"),
                 InstigatorType.SCHEDULE,
@@ -386,7 +443,7 @@ def test_get_unloadable_job(graphql_context):
             )
         )
 
-        instance.add_job_state(
+        instance.add_instigator_state(
             InstigatorState(
                 _get_unloadable_schedule_origin("unloadable_stopped"),
                 InstigatorType.SCHEDULE,
@@ -404,3 +461,119 @@ def test_get_unloadable_job(graphql_context):
         result.data["unloadableInstigationStatesOrError"]["results"][0]["name"]
         == "unloadable_running"
     )
+
+
+def test_future_ticks_until(graphql_context):
+    schedule_selector = infer_schedule_selector(graphql_context, "timezone_schedule")
+
+    future_ticks_start_time = create_pendulum_time(2019, 2, 27, tz="US/Central").timestamp()
+
+    # Start a single schedule, future tick run requests only available for running schedules
+    start_result = execute_dagster_graphql(
+        graphql_context,
+        START_SCHEDULES_QUERY,
+        variables={"scheduleSelector": schedule_selector},
+    )
+    assert (
+        start_result.data["startSchedule"]["scheduleState"]["status"]
+        == InstigatorStatus.RUNNING.value
+    )
+
+    future_ticks_start_time = create_pendulum_time(2019, 2, 27, tz="US/Central").timestamp()
+    future_ticks_end_time = create_pendulum_time(2019, 3, 2, tz="US/Central").timestamp()
+
+    result = execute_dagster_graphql(
+        graphql_context,
+        GET_SCHEDULE_FUTURE_TICKS_UNTIL,
+        variables={
+            "scheduleSelector": schedule_selector,
+            "ticksAfter": future_ticks_start_time,
+            "ticksUntil": future_ticks_end_time,
+        },
+    )
+
+    future_ticks = result.data["scheduleOrError"]["futureTicks"]
+
+    assert future_ticks
+    assert len(future_ticks["results"]) == 3
+
+    timestamps = [future_tick["timestamp"] for future_tick in future_ticks["results"]]
+
+    assert timestamps == [
+        create_pendulum_time(2019, 2, 27, tz="US/Central").timestamp(),
+        create_pendulum_time(2019, 2, 28, tz="US/Central").timestamp(),
+        create_pendulum_time(2019, 3, 1, tz="US/Central").timestamp(),
+    ]
+
+
+def test_repository_batching(graphql_context):
+    traced_counter.set(Counter())
+    selector = infer_repository_selector(graphql_context)
+    result = execute_dagster_graphql(
+        graphql_context,
+        REPOSITORY_SCHEDULES_QUERY,
+        variables={"repositorySelector": selector},
+    )
+    assert result.data
+    assert "repositoryOrError" in result.data
+    assert "schedules" in result.data["repositoryOrError"]
+    counter = traced_counter.get()
+    counts = counter.counts()
+    assert counts
+    assert len(counts) == 2
+
+    # We should have a single batch call to fetch run records (to fetch schedule runs) and a single
+    # batch call to fetch instigator state, instead of separate calls for each schedule (~18
+    # distinct schedules in the repo)
+    # 1) `get_run_records` is fetched to instantiate GrapheneRun
+    # 2) `all_instigator_state` is fetched to instantiate GrapheneSchedule
+    assert counts.get("DagsterInstance.get_run_records") == 1
+    assert counts.get("DagsterInstance.all_instigator_state") == 1
+
+
+def test_start_schedule_with_default_status(graphql_context):
+    schedule_selector = infer_schedule_selector(graphql_context, "running_in_code_schedule")
+
+    result = execute_dagster_graphql(
+        graphql_context,
+        GET_SCHEDULE_STATE_QUERY,
+        variables={"scheduleSelector": schedule_selector},
+    )
+
+    schedule_origin_id = result.data["scheduleOrError"]["scheduleState"]["id"]
+    assert result.data["scheduleOrError"]["scheduleState"]["status"] == "RUNNING"
+
+    # Start a single schedule
+    start_result = execute_dagster_graphql(
+        graphql_context,
+        START_SCHEDULES_QUERY,
+        variables={"scheduleSelector": schedule_selector},
+    )
+
+    assert (
+        "You have attempted to start schedule running_in_code_schedule, but it is already running"
+        in start_result.data["startSchedule"]["message"]
+    )
+
+    # Stop a single schedule
+    stop_result = execute_dagster_graphql(
+        graphql_context,
+        STOP_SCHEDULES_QUERY,
+        variables={"scheduleOriginId": schedule_origin_id},
+    )
+    assert (
+        stop_result.data["stopRunningSchedule"]["scheduleState"]["status"]
+        == InstigatorStatus.STOPPED.value
+    )
+
+    # Start a single schedule
+    start_result = execute_dagster_graphql(
+        graphql_context,
+        START_SCHEDULES_QUERY,
+        variables={"scheduleSelector": schedule_selector},
+    )
+
+    assert (
+        start_result.data["startSchedule"]["scheduleState"]["status"]
+        == InstigatorStatus.RUNNING.value
+    )
